<?php

	/**
	 * Get the main category of a post (usefull when we have sub categories).
	 */
	function wpgrade_main_category($category, $shout = true) {
		if ( ! empty($category)) {
			$parent = get_cat_name($category[0]->category_parent);
			if ( ! empty($parent)) {
				if ($shout) {
					echo strtolower($parent);
				}
				else { // ! shout
					return strtolower($parent);
				}
			}
			else {
				if ($shout) {
					echo strtolower($category[0]->cat_name);
				}
				else { // ! shout
					return strtolower($category[0]->cat_name);
				}
			}
		}
		else { // empty $category
			return '';
		}
	}

	/**
	 * The following code is inspired by Yoast SEO.
	 */
	function wpgrade_get_current_canonical_url() {
		global $wp_query;

		if ($wp_query->is_404 || $wp_query->is_search) {
			return false;
		}

		$haspost = count($wp_query->posts) > 0;

		if (get_query_var('m')) {
			$m = preg_replace('/[^0-9]/', '', get_query_var('m'));
			switch (strlen($m)) {
				case 4:
					$link = get_year_link( $m );
					break;
				case 6:
					$link = get_month_link(substr($m, 0, 4), substr($m, 4, 2));
					break;
				case 8:
					$link = get_day_link(substr($m, 0, 4), substr($m, 4, 2), substr($m, 6, 2));
					break;
				default:
					return false;
			}
		}
		elseif (($wp_query->is_single || $wp_query->is_page) && $haspost) {
			$post = $wp_query->posts[0];
			$link = get_permalink(wpgrade::lang_post_id($post->ID));
		}
		elseif ($wp_query->is_author && $haspost) {
			$author = get_userdata(get_query_var('author'));
			if ($author === false) {
				return false;
			}
			$link = get_author_posts_url($author->ID, $author->user_nicename);
		}
		elseif ($wp_query->is_category && $haspost) {
			$link = get_category_link(get_query_var('cat'));
		}
		elseif ($wp_query->is_tag && $haspost) {
			$tag = get_term_by('slug', get_query_var('tag'), 'post_tag');
			if ( ! empty($tag->term_id)) {
				$link = get_tag_link($tag->term_id);
			}
		}
		elseif ($wp_query->is_day && $haspost) {
			$link = get_day_link
				(
					get_query_var('year'),
					get_query_var('monthnum'),
					get_query_var('day')
				);
		}
		elseif ($wp_query->is_month && $haspost) {
			$link = get_month_link
				(
					get_query_var('year'),
					get_query_var('monthnum')
				);
		}
		elseif ($wp_query->is_year && $haspost) {
			$link = get_year_link(get_query_var('year'));
		}
		elseif ($wp_query->is_home) {
			if ((get_option('show_on_front') == 'page') && ($pageid = get_option('page_for_posts'))) {
				$link = get_permalink($pageid);
			}
			else {
				if (function_exists('icl_get_home_url')) {
					$link = icl_get_home_url();
				}
				else { // icl_get_home_url does not exist
					$link = home_url();
				}
			}
		}
		elseif ($wp_query->is_tax && $haspost) {
			$taxonomy = get_query_var('taxonomy');
			$term = get_query_var('term');
			$link = get_term_link($term, $taxonomy);
		}
		elseif ($wp_query->is_archive && function_exists('get_post_type_archive_link') && ($post_type = get_query_var('post_type'))) {
			$link = get_post_type_archive_link( $post_type );
		}
		else {
			return false;
		}

		//let's see about the page number
		$page = get_query_var('page');
		if (empty($page )) {
			$page = get_query_var('paged');
		}

		if ( ! empty($page) && $page > 1) {
			$link = trailingslashit($link) ."page/$page";
			$link = user_trailingslashit($link, 'paged');
		}

		return $link;
	}
